@@ -1,5 +1,4 @@
 use alloc::boxed::Box;
-use alloc::sync::Arc;
 use alloc::vec::Vec;
 use core::fmt::Debug;
 
@@ -8,8 +7,10 @@ use zeroize::Zeroize;
 
 use crate::msgs::ffdhe_groups::FfdheGroup;
 use crate::sign::SigningKey;
+use crate::sync::Arc;
 pub use crate::webpki::{
-    verify_tls12_signature, verify_tls13_signature, WebPkiSupportedAlgorithms,
+    verify_tls12_signature, verify_tls13_signature, verify_tls13_signature_with_raw_key,
+    WebPkiSupportedAlgorithms,
 };
 #[cfg(all(doc, feature = "tls12"))]
 use crate::Tls12CipherSuite;
@@ -483,6 +484,87 @@ pub trait ActiveKeyExchange: Send + Sync {
         Ok(complete_res)
     }
 
+    /// For hybrid key exchanges, returns the [`NamedGroup`] and key share
+    /// for the classical half of this key exchange.
+    ///
+    /// There is no requirement for a hybrid scheme (or any other!) to implement
+    /// `hybrid_component()`. It only enables an optimization; described below.
+    ///
+    /// "Hybrid" means a key exchange algorithm which is constructed from two
+    /// (or more) independent component algorithms. Usually one is post-quantum-secure,
+    /// and the other is "classical".  See
+    /// <https://datatracker.ietf.org/doc/draft-ietf-tls-hybrid-design/11/>
+    ///
+    /// # Background
+    /// Rustls always sends a presumptive key share in its `ClientHello`, using
+    /// (absent any other information) the first item in [`CryptoProvider::kx_groups`].
+    /// If the server accepts the client's selection, it can complete the handshake
+    /// using that key share.  If not, the server sends a `HelloRetryRequest` instructing
+    /// the client to send a different key share instead.
+    ///
+    /// This request costs an extra round trip, and wastes the key exchange computation
+    /// (in [`SupportedKxGroup::start()`]) the client already did.  We would
+    /// like to avoid those wastes if possible.
+    ///
+    /// It is early days for post-quantum-secure hybrid key exchange deployment.
+    /// This means (commonly) continuing to offer both the hybrid and classical
+    /// key exchanges, so the handshake can be completed without a `HelloRetryRequest`
+    /// for servers that support the offered hybrid or classical schemes.
+    ///
+    /// Implementing `hybrid_component()` enables two optimizations:
+    ///
+    /// 1. Sending both the hybrid and classical key shares in the `ClientHello`.
+    ///
+    /// 2. Performing the classical key exchange setup only once.  This is important
+    ///    because the classical key exchange setup is relatively expensive.
+    ///    This optimization is permitted and described in
+    ///    <https://www.ietf.org/archive/id/draft-ietf-tls-hybrid-design-11.html#section-3.2>
+    ///
+    /// Both of these only happen if the classical algorithm appears separately in
+    /// the client's [`CryptoProvider::kx_groups`], and if the hybrid algorithm appears
+    /// first in that list.
+    ///
+    /// # How it works
+    /// This function is only called by rustls for clients.  It is called when
+    /// constructing the initial `ClientHello`.  rustls follows these steps:
+    ///
+    /// 1. If the return value is `None`, nothing further happens.
+    /// 2. If the given [`NamedGroup`] does not appear in
+    ///    [`CryptoProvider::kx_groups`], nothing further happens.
+    /// 3. The given key share is added to the `ClientHello`, after the hybrid entry.
+    ///
+    /// Then, one of three things may happen when the server replies to the `ClientHello`:
+    ///
+    /// 1. The server sends a `HelloRetryRequest`.  Everything is thrown away and
+    ///    we start again.
+    /// 2. The server agrees to our hybrid key exchange: rustls calls
+    ///    [`ActiveKeyExchange::complete()`] consuming `self`.
+    /// 3. The server agrees to our classical key exchange: rustls calls
+    ///    [`ActiveKeyExchange::complete_hybrid_component()`] which
+    ///    discards the hybrid key data, and completes just the classical key exchange.
+    fn hybrid_component(&self) -> Option<(NamedGroup, &[u8])> {
+        None
+    }
+
+    /// Completes the classical component of the key exchange, given the peer's public key.
+    ///
+    /// This is only called if `hybrid_component` returns `Some(_)`.
+    ///
+    /// This method must return an error if `peer_pub_key` is invalid: either
+    /// mis-encoded, or an invalid public key (such as, but not limited to, being
+    /// in a small order subgroup).
+    ///
+    /// The shared secret is returned as a [`SharedSecret`] which can be constructed
+    /// from a `&[u8]`.
+    ///
+    /// See the documentation on [`Self::hybrid_component()`] for explanation.
+    fn complete_hybrid_component(
+        self: Box<Self>,
+        _peer_pub_key: &[u8],
+    ) -> Result<SharedSecret, Error> {
+        unreachable!("only called if `hybrid_component()` implemented")
+    }
+
     /// Return the public key being used.
     ///
     /// For ECDHE, the encoding required is defined in
@@ -523,7 +605,7 @@ pub struct CompletedKeyExchange {
     pub secret: SharedSecret,
 }
 
-/// The result from [`ActiveKeyExchange::complete`].
+/// The result from [`ActiveKeyExchange::complete`] or [`ActiveKeyExchange::complete_hybrid_component`].
 pub struct SharedSecret {
     buf: Vec<u8>,
     offset: usize,
@@ -565,6 +647,12 @@ impl From<&[u8]> for SharedSecret {
     }
 }
 
+impl From<Vec<u8>> for SharedSecret {
+    fn from(buf: Vec<u8>) -> Self {
+        Self { buf, offset: 0 }
+    }
+}
+
 /// This function returns a [`CryptoProvider`] that uses
 /// FIPS140-3-approved cryptography.
 ///
@@ -607,14 +695,14 @@ pub fn default_fips_provider() -> CryptoProvider {
 mod static_default {
     #[cfg(not(feature = "std"))]
     use alloc::boxed::Box;
-    use alloc::sync::Arc;
+    #[cfg(feature = "std")]
+    use std::sync::OnceLock;
 
     #[cfg(not(feature = "std"))]
     use once_cell::race::OnceBox;
-    #[cfg(feature = "std")]
-    use once_cell::sync::OnceCell;
 
     use super::CryptoProvider;
+    use crate::sync::Arc;
 
     #[cfg(feature = "std")]
     pub(crate) fn install_default(
@@ -637,7 +725,7 @@ mod static_default {
     }
 
     #[cfg(feature = "std")]
-    static PROCESS_DEFAULT_PROVIDER: OnceCell<Arc<CryptoProvider>> = OnceCell::new();
+    static PROCESS_DEFAULT_PROVIDER: OnceLock<Arc<CryptoProvider>> = OnceLock::new();
     #[cfg(not(feature = "std"))]
     static PROCESS_DEFAULT_PROVIDER: OnceBox<Arc<CryptoProvider>> = OnceBox::new();
 }
